/*----------------------------------------------------------------------*/
/*                                                         		*/
/* Module Name: mathproxy						*/
/*                                                         		*/
/* Module Description: proxy server for MathLink programs		*/
/*                                                         		*/
/*----------------------------------------------------------------------*/

/*#define _RUN_FG*/
#define _MAXBUFFER		16384
#define _MATHLINK_WKA		9090
#define _TIMEOUT		3600
#define logfile			"/var/log/mathprox"


/*----------------------------------------------------------------------*/
/*                                                         		*/
/* Includes: 								*/
/*                                                        		*/
/*----------------------------------------------------------------------*/
#ifndef __EXTENSIONS__
#define __EXTENSIONS__
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/resource.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <sys/sockio.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <netdir.h>
#include <unistd.h>
#include <tiuser.h>
#include <signal.h>

   char 	MLprog[256] = "";	/* math link program */
   int 		MLport = 0;		/* math link port */
   char		MLaccess[256] = "";	/* access list file */
   char		server[256] = "";	/* server name */
   int		datarecv = 0;		/* number of bytes receuved */
   int		datasent = 0;		/* number of bytes sent back */


/*----------------------------------------------------------------------*/
/*                                                         		*/
/* Internal Procedure Name: mathproxy					*/
/*                                                         		*/
/* Procedure Description: this is the MathLink proxy server		*/
/*    It forwards MathLink packets between a Mathematica client and	*/
/*    the MathLink service program.	                    		*/
/*                                                         		*/
/* Procedure Arguments: incoming socket, priority			*/
/*                                                         		*/
/* Procedure Returns: -1 on error					*/
/*                                                         		*/
/*----------------------------------------------------------------------*/
extern "C"
   void QuitMLprog (int) 
   {
      printf ("received %i kB / transmitted %i kB (%s)\n",
             (datarecv + 512) / 1024, (datasent + 512) / 1024, server);
      exit (0);
   }


extern "C"
   void BrokenConnection (int)
   {
      printf ("received %i kB / transmitted %i kB (%s)\n",
             (datarecv + 512) / 1024, (datasent + 512) / 1024, server);
      exit (0);
   }

extern "C"
   void oob (int)
   {
      printf ("received OOB\n");
      // printf ("received %i kB / transmitted %i kB (%s)\n",
             // (datarecv + 512) / 1024, (datasent + 512) / 1024, server);
      // exit (0);
   }



   static int mathproxy (int insock, int priority)
   {
      int	pid;		/* child process id */
      int	port = -1;	/* port of output socket */
      int	outsock;	/* output socket */
      struct sockaddr_in name;	/* server address */
      fd_set	port_set;	/* read set for select */
      //fd_set	oob_set;	/* excpetion set for select */
      char*	buf;		/* temp buffer */
      socklen_t	num;		/* number of bytes in buffer */
      struct timespec	wait = 	/* wait tick 100ms */
         {0, 100000000};
      int	sel;		/* return of select */
      struct timeval	timeout;/* timeout for select */
   
   //    /* set keep alive of input socket */
      // sigset (SIGPIPE, BrokenConnection);
      // mark = 1;
      // if (setsockopt (insock, SOL_SOCKET, SO_KEEPALIVE, (char*) &mark, 
                     // sizeof (mark)) == -1) {
         // printf ("error setting socket keep alive (errno %i)\n", errno);
      // }
      // sigset (SIGURG, oob);
      // pid = -getpid();
      // if (ioctl (insock, SIOCSPGRP, (char*) &pid) < 0) {
         // printf ("error setting OOB interrupt (errno %i)\n", errno);
         // close (insock);
         // return -1;
      // }
   
      /* create output socket */
      outsock = socket (PF_INET, SOCK_STREAM, 0);
      if (outsock < 0) {
         printf ("error creating socket (errno %i)\n", errno);
         close (insock);
         return -2;
      }
      /* bind socket */
      name.sin_family = AF_INET;
      name.sin_port = htons (0);
      name.sin_addr.s_addr = htonl (INADDR_ANY);
      if (bind (outsock, (struct sockaddr*) &name, 
               sizeof (name)) < 0) {
         printf ("error binding socket (errno %i)\n", errno);
         close (outsock);
         close (insock);
         return -3;
      }
      num = sizeof (name);
      if (getsockname (outsock, (struct sockaddr*) &name, 
                      &num) < 0) {
         printf ("error obtaining socket name (errno %i)\n", errno);
         close (outsock);
         close (insock);
         return -4;
      }
      port = ntohs (name.sin_port);
      close (outsock);
   
      /* setup child watch */
      sigset (SIGCHLD, QuitMLprog);
   
      /* start MathLink program */
      pid = fork ();
      if (pid == -1) {
         printf ("error while forking\n");
         close (insock);
         return -5;
      }
      /* start MathLink program if child */
      else if (pid == 0) {
         char 		portnum[32];	/* port number */
         char 		progname[256];	/* program name */
         char*		p;		/* pointer to program name */
         sprintf (portnum, "%i", port);
         if ((p = strrchr (MLprog, '/')) == 0) {
            strcpy (progname, MLprog);
         }
         else {
            strcpy (progname, p+1);
         }
         execl (MLprog, progname, "-linkcreate", "-linkprotocol", "tcp",
               "-linkname", portnum, 0);
         printf ("Cannot start %s\n", MLprog);
         return 1;
      }
   
      /* try to open connection */
      nanosleep (&wait, 0);
      for (num = 0;;) {
         /* create connection */
         outsock = socket (PF_INET, SOCK_STREAM, 0);
         if (outsock < 0) {
            printf ("error creating socket (errno %i)\n", errno);
            close (insock);
            return -6;
         }
         /* connect to socket */
         name.sin_family = AF_INET;
         name.sin_port = htons (port);
         name.sin_addr.s_addr = htonl (0);
         if (connect (outsock, (struct sockaddr*) &name, 
                     sizeof (name)) == 0) {
            break;
         }
         close (outsock);
         if (num == 50) {
            printf ("error opening socket (errno %i)\n", errno);
            close (insock);
            return -7;
         }
         num++;
         nanosleep (&wait, 0);
      }
   
      /* allocate message buffer */
      buf = (char*) malloc (_MAXBUFFER);
      if (buf == 0) {
         printf ("error allocating memory\n");
         close (outsock);
         close (insock);
         return -1;
      }
   
      /* exchange data between sockets */
      for (;;) {
         /* setup for select */   
         FD_ZERO (&port_set);
         FD_SET (insock, &port_set);
         FD_SET (outsock, &port_set);
         // FD_ZERO (&oob_set);
         // FD_SET (insock, &oob_set);
         // FD_SET (outsock, &oob_set);
      
         /* wait for input from either socket */
         timeout.tv_sec = _TIMEOUT;
         timeout.tv_usec = 0;
         sel = select (FD_SETSIZE, &port_set, 0, 0, &timeout);
         /* check err */
         if (sel == -1) {
            printf ("select failed (errno %i)\n", errno);
         }
         /* check timeout */
         else if (sel == 0) {
            BrokenConnection (0);
         }
         /* sockets have data  */
         else if (sel > 0) {
         //    /* copies out-of-band mark mark from in to out */
            // if (FD_ISSET (insock, &oob_set)) {
            //    /* discard data before oob mark */
               // printf ("Received OOB siganl from in\n");
               // for (;;) {
                  // if (ioctl (insock, SIOCATMARK, &mark) == -1) {
                     // printf ("OOB mark error (errno %i)\n", errno);
                     // continue;
                  // }
                  // if (mark) {
                     // break;
                  // }
                  // num = read (insock, buf, _MAXBUFFER);    
               // }
            //    /* read oob mark */
               // num = recv (insock, (char*)&mark, 1, MSG_OOB);
            //    /* forward oob mark to out */
               // if (num > 0) {
                  // printf ("received OOB siganl from in\n");
                  // send (outsock, (char*)&mark, 1, MSG_OOB);
               // }
            // }
            // /* copies out-of-band mark from out to in */
            // else if (FD_ISSET (outsock, &oob_set)) {
               // printf ("received OOB siganl from out\n");
            // }
            /* copies data from in to out */
            if (FD_ISSET (insock, &port_set)) {
               num = read (insock, buf, _MAXBUFFER);
               /* printf ("received %i bytes from in\n", num);*/
               if (num > 0) {
                  datarecv += num;
                  write (outsock, buf, num);
               }
               /* connection broken */
               else {
                  BrokenConnection (0);
               }
            }
            /* copies data from out to in */
            else if (FD_ISSET (outsock, &port_set)) {
               num = read (outsock, buf, _MAXBUFFER);
               /* printf ("received %i bytes from out\n", num); */
               if (num > 0) {
                  datasent += num;
                  write (insock, buf, num);
               }
               /* connection broken */
               else {
                  BrokenConnection (0);
               }
            }
         }
      }
   
      close (outsock);
      close (insock);
      free (buf);
      return 0;
   }


/*----------------------------------------------------------------------*/
/*                                                         		*/
/* Internal Procedure Name: checkAccess					*/
/*                                                         		*/
/* Procedure Description: this function checks the access rights of	*/
/*                        supplied socket	            		*/
/*                                                         		*/
/* Procedure Arguments: incoming socket					*/
/*                                                         		*/
/* Procedure Returns: 0 - denied, 1 - allowed (low priority)		*/
/*    2 - allowed (medium priority), 3 - allowed (high priority)	*/
/*                                                         		*/
/*----------------------------------------------------------------------*/
   static int checkAccess (int sock) 
   {
      int		priority;	/* access priority */
      struct sockaddr_in name;		/* server address */
      socklen_t		size;		/* socket size */
      time_t		now;		/* time now */
   
      /* get server address */
      size = sizeof (name);
      if (getpeername (sock, (struct sockaddr*) &name, &size) < 0) {
         return 0;
      }
      strcpy (server, inet_ntoa (name.sin_addr));
   
      priority = 2;
   
      now = time (0);
      printf ("accpeted connection from %s (priority %i) on %s",
             server, priority, ctime (&now));
      fflush (stdout);
   
      return priority;
   }


/*----------------------------------------------------------------------*/
/*                                                         		*/
/* Internal Procedure Name: parseArguments				*/
/*                                                         		*/
/* Procedure Description: parse the cmd line arguments			*/
/*                                                         		*/
/* Procedure Arguments: argc and argv					*/
/*                                                         		*/
/* Procedure Returns: 0 - success, -1 otherwise				*/
/*                                                         		*/
/*----------------------------------------------------------------------*/
   static int parseArguments (int argc, char *argv[])
   {
      int 		c;
      extern char*	optarg;
      extern int	optind;
      int		errflag = 0;
   
      strcpy (MLprog, "");
      MLport = _MATHLINK_WKA;
      strcpy (MLaccess, "");
      while ((c = getopt (argc, argv, "p:a:")) != EOF) {
         switch (c) {
            /* port number */
            case 'p':
               {
                  MLport = atoi (optarg);
                  break;
               }
            /* access list */
            case 'a':
               {
                  strcpy (MLaccess, optarg);
                  break;
               }
            case '?':
               {
                  errflag = 1;
                  break;
               }
         }
      }
   
      if (!errflag && (optind < argc)) {
         strcpy (MLprog, argv[optind]);
      }
   
      if (errflag || (strlen (MLprog) == 0)) {
         fprintf (stderr, "usage: mathproxy -p port "
                 "-a access_file mathlink_prog\n");
         exit (2);
      }
      return 0;
   }



/*----------------------------------------------------------------------*/
/*                                                         		*/
/* Internal Procedure Name: checkStdInHandle				*/
/*                                                         		*/
/* Procedure Description: check FD 0 to determine if it was called by	*/
/*			  a port monitor				*/
/*                                                         		*/
/* Procedure Arguments: void						*/
/*                                                         		*/
/* Procedure Returns: 1 if called by port mon., 0 if not, -1 if failed	*/
/*                                                         		*/
/*----------------------------------------------------------------------*/
   static int checkStdInHandle (void)
   {
      struct sockaddr_in saddr;
      socklen_t		asize = sizeof (saddr);
      int 		rpcfdtype;
      socklen_t 	ssize = sizeof (int);
   
      /* get the socket address of FD 0 */
      if (getsockname (0, (struct sockaddr*) &saddr, &asize) == 0) {
      
         /* check whether valid internet address */
         if (saddr.sin_family != AF_INET) {
            return -1;
         }
      	 /* get socket type: tcp only */
         if ((getsockopt (0, SOL_SOCKET, SO_TYPE,
                         (char*) &rpcfdtype, &ssize) == -1) || 
            (rpcfdtype != SOCK_STREAM)) {
            return -1;
         }
         return 1;
      }
      else {
         return 0;
      }
   }


/*----------------------------------------------------------------------*/
/*                                                         		*/
/* Main Program								*/
/*                                                         		*/
/* Description: installs a listening socket and calls mathproxy	for	*/
/* 		every new connection; if started by port monitor just	*/
/* 		calls the mathproxy					*/
/* 									*/
/*----------------------------------------------------------------------*/
extern "C"
   void reaper (int) 
   {
      int 		options;
      int 		error;
      siginfo_t		info;
   
      options = WNOHANG | WEXITED;
      memset ((char*) &info, 0, sizeof (info));
      error = waitid (P_ALL, 0, &info, options);
   }


   int main (int argc, char *argv[])
   {
      /* parse command line arguments */
      if (parseArguments (argc, argv) != 0) {
         return -1;
      }
   
      /* test if started by a port monitor */
      switch (checkStdInHandle()) {
         /* port monitor */
         case 1: 
            {
               int		i;		/* file descriptor */
               int		priority;	/* access right */
            
               /* check access rights */
               priority = checkAccess (0);
               if (priority <= 0) {
                  return -1;
               }
            
               /* make sure stdout/stderr are not used */
               i = open (logfile, O_CREAT | O_WRONLY | O_APPEND);
               if (i < 0) {
                  i = open ("/dev/null", 2);
               }
               (void) dup2(i, 1);
               (void) dup2(i, 2);
            
               return mathproxy (0, priority);
            }
         /* stand-alone */
         case 0: 
            {
               int		pid;	/* process id */
               int		sock; 	/* socket */
               struct sockaddr_in name;	/* server address */
            
               printf ("Start MathLink proxy server\n");
            
               /* fork */
            #ifndef _RUN_FG
               pid = fork ();
               if (pid < 0) {
                  printf ("error while forking\n");
                  return 1;
               }
               /* return if parent */
               if (pid > 0) {
                  return 0;
               }
               /* child process continues */
               setsid ();
            #endif
            
               /* setup child watch */
               sigset (SIGCHLD, reaper);
            
               /* create listening socket */
               sock = socket (PF_INET, SOCK_STREAM, 0);
               if (sock < 0) {
                  printf ("error creating socket\n");
                  return -1;
               }
               /* bind socket */
               name.sin_family = AF_INET;
               name.sin_port = htons (MLport);
               name.sin_addr.s_addr = htonl (INADDR_ANY);
               if (bind (sock, (struct sockaddr*) &name, 
                        sizeof (name)) < 0) {
                  close (sock);
                  printf ("error binding socket\n");
                  return -2;
               }
            
               while (1) {
                  int		c_sock;		/* client socket */
                  struct sockaddr_in c_addr;	/* client address */
                  socklen_t	c_len;		/* client addr length */
                  int		priority;	/* access right */
               
                  /* listen at well-known address */
                  if (listen (sock, 1) < 0) {
                     printf ("error while listening to socket\n");
                     return -3;
                  }
               
                  /* accept connection */
                  c_len = sizeof (c_addr);
                  c_sock = accept (sock, (struct sockaddr*) &c_addr, 
                                  &c_len);
                  if (c_sock == -1) {
                     /* continue on signal */
                     if (errno == EINTR) {
                        continue;
                     }
                     else {
                        printf ("error while accepting connection "
                               "(errno %i)\n", errno);
                        return -4;
                     }
                  }
               
                  /* check access rights */
                  priority = checkAccess (c_sock);
                  if (priority <= 0) {
                     close (c_sock);
                     continue;
                  }
               
                  /* fork new child for each accepted connection */
                  pid = fork ();
                  if (pid < 0) {
                     printf ("error while forking\n");
                     return -6;
                  }
                  /* call message server if child */
                  if (pid == 0) {
                     setsid ();
                     return mathproxy (c_sock, priority);
                  }
                  /* continue listening if parent */
               }
            }
         /* error */
         default: 
            {
               printf ("illegale socket\n");
               return -1;
            }
      }
      /* no reason to end up here */
      exit (1);
   }
