#ifndef __EXTENSIONS__
#define __EXTENSIONS__
#endif
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/resource.h>

#if defined(linux)
#include <sys/sysinfo.h>
#define RLIMIT_VMEM RLIMIT_AS
#elif defined(__sun)
#include <sys/swap.h>
#include <sys/processor.h>

   struct swaptable_t {
      int swt_n;
      struct swapent swt_ent[32];
   };
   struct swaptable_t swptable;
#elif defined(__CYGWIN__)
#define RLIMIT_VMEM RLIMIT_AS
#endif

   int main(void)
   {
   #if defined(linux)
   #elif defined(__sun)
      processor_info_t pinfo;
      const char* p;
      unsigned long swmem, swfree, vtmem, vtfree, i;
   #endif
      unsigned long n, psize, phmem, avmem;
      int size;
      struct rlimit lim;
   
      psize=sysconf(_SC_PAGESIZE);
      phmem=sysconf(_SC_PHYS_PAGES);
      avmem=sysconf(_SC_AVPHYS_PAGES);
      n=sysconf(_SC_NPROCESSORS_CONF);
   
   
      printf("\nPage size: %5ld bytes\n\n",psize);
      size = (long long)phmem * psize / 1024 / 1024; 
      printf("Total physical memory: %5d Mbytes\n", size);
      size = (long long)avmem * psize / 1024 / 1024; 
      printf("Free  physical memory: %5d Mbytes\n", size);

   #if !defined(linux) && defined(__sun)   
      swptable.swt_n=swapctl(SC_GETNSWP,0);
      for( i=0 ; i<swptable.swt_n ; i++ ) {
	 swptable.swt_ent[i].ste_path = malloc (256);
      }
      swapctl(SC_LIST,&swptable);
      for( swmem=0, swfree=0, i=0 ; i<swptable.swt_n ; i++ ) {
         swmem+=swptable.swt_ent[i].ste_pages;
	 swfree+=swptable.swt_ent[i].ste_free;
      }
      vtmem=phmem+swmem;
      vtfree=avmem+swfree;
      size = (long long)swmem * psize / 1024 / 1024; 
      printf("Total disk swap space: %5d Mbytes\n", size);
      size = (long long)swfree * psize / 1024 / 1024; 
      printf("Free  disk swap space: %5d Mbytes\n", size);
      size = (long long)vtmem * psize / 1024 / 1024; 
      printf("Total virtual  memory: %5d Mbytes\n", size);
      size = (long long)vtfree * psize / 1024 / 1024; 
      printf("Free  virtual  memory: %5d Mbytes\n", size);
   #endif   
   #if !defined(linux) && defined(__sun)
      printf ("\nSwap files:\n");
      for( i=0 ; i<swptable.swt_n ; i++ ) {
	 p = swptable.swt_ent[i].ste_path;
         printf ("  %s\n", p ? p : "default");
      }
   #endif
      
   #if defined(linux)
      printf ("\nCPUs: %i (available %i)\n",
              get_nprocs_conf(), get_nprocs());
   #elif defined(__sun)
      printf("\n #   CPU      FPU               Speed   Status\n");
      for( i=0 ; i<32 ; i++ ) {
         if (processor_info(i,&pinfo) == 0) {
            printf("%2lu  %-8s %-16s %4d MHz  %s\n",
                  i ,pinfo.pi_processor_type,
                  pinfo.pi_fputypes, pinfo.pi_clock,
                  (pinfo.pi_state==P_OFFLINE) ? "Offline":
                  ((pinfo.pi_state==P_ONLINE) ? "Online":"PowerOff" ) );
         }
      }
   #endif
   
      printf("\n");
      getrlimit (RLIMIT_CORE, &lim);
      printf ("Maximum core: %li kB (hard %li MB)\n", 
         lim.rlim_cur/1024, lim.rlim_max/1024/1024);
      getrlimit (RLIMIT_DATA, &lim);
      printf ("Maximum memory: %li MB (hard %li MB)\n", 
         lim.rlim_cur/1024/1024, lim.rlim_max/1024/1024);
      getrlimit (RLIMIT_FSIZE, &lim);
      printf ("Maximum file size: %li MB (hard %li MB)\n", 
         lim.rlim_cur/1024/1024, lim.rlim_max/1024/1024);
      getrlimit (RLIMIT_STACK, &lim);
      printf ("Maximum stack size: %li MB (hard %li MB)\n", 
         lim.rlim_cur/1024/1024, lim.rlim_max/1024/1024);
      getrlimit (RLIMIT_VMEM, &lim);
      printf ("Maximum virtual memory: %li MB (hard %li MB)\n", 
         lim.rlim_cur/1024/1024, lim.rlim_max/1024/1024);
      
      printf("\n");
      exit(0);
      return 0;
   }
