#ifndef CHANNELINDEX_HH
#define CHANNELINDEX_HH

#include <string>

/**  ChannelIndex list entry for MultiDacc.
  *  ChannelIndex holds Dacc index information.
  */
class ChannelIndex
{
public:
  typedef unsigned int index_type;

  //------------------------------------  Constructor/Destructor.
public:
  /**  Default Constructor.
    */
  ChannelIndex(void);

  /**  @memo Construct a channelindex object.
    *  Construct an object given a channel name, 
    *  an output pointer address and a channel type.
    */
  ChannelIndex(const char* name);

  ChannelIndex(const char* name, int index);

  /**  Destroy a channel entry.
    */
  ~ChannelIndex();

  //------------------------------------  Accessors
  /**  @memo Compare the name.
    *  EqName() returns true if the channel name is the same as the argument.
    */
  bool EqName(const char *name) const;

  bool getDebug(void) const;

  /**  @memo Get the decimation factor.
    *  getIndex() returns the decimation factor.
    */
  index_type getIndex(void) const;

  /**  @memo Get the channel name.
    *  getName() returns a pointer to the channel name string.
    */
  const char* getName(void) const;

  /**  Set/clear the debug flag.
    *  The debug flag is set as specified.
    */
  void setDebug(bool yorn) {mDebug = yorn;}
  void setIndex(index_type index) {mIndex = index;}

private:                                // Private data members.
  /**  Channel Name.
    *  The name of the channel to be processed.
    */
  std::string mName;

  /**  Decimation Factor.
    *  Decim8 contains the number of raw samples that must be averaged 
    *  together for a single TSeries entry.
    */
  index_type mIndex;

  /**  Debug flag.
    *  If set, copious error and status messages are printed to cerr.
    */
  bool mDebug;
};

inline bool 
ChannelIndex::EqName(const char *name) const {
    return (mName == name);
}

inline bool 
ChannelIndex::getDebug(void) const {
    return mDebug;
}

inline ChannelIndex::index_type 
ChannelIndex::getIndex(void) const {
    return mIndex;
}

inline const char* 
ChannelIndex::getName(void) const {
    return mName.c_str();
}

#endif // CHANNELINDEX_HH
