#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <sys/time.h>
#include <string.h>
#include "CmMessage.h"
#include "FrameL.h"


/*----------------------------------------------------- FdPutFrameInMessage---*/
int FdPutFrameInMessage (CmMessage message, FrameH *frame, int compress)
/*----------------------------------------------------------------------------*/
{
  int nBytes;
  static char *buf;
  static int bufSize = 0;

  if (message == NULL) return(0);
  if (frame   == NULL) return(0);

  /*----------------------- Allocate the internal buffer for the first call---*/
  if (buf == NULL) {
    bufSize = 540000;
    buf     = (char *) malloc(bufSize);
    if (buf == NULL) return(0);}

  /*------Write the frame into the local buffer. If the buffer is too 
    small, increase its size (without limit) until the frame fits-------------*/
  do {
    nBytes = FrameWriteToBuf(frame, compress, buf, bufSize);

    if (nBytes == 0) {
      bufSize = 2 * bufSize;
      free(buf);
      buf = malloc(bufSize);
      if(buf == NULL) return(0);}
  } 
  while (nBytes == 0);

  /*------------------------------------------------- build the Cm message ---*/
  CmMessagePutText     (message, frame->name);
  CmMessagePutInt      (message, frame->run);
  CmMessagePutInt      (message, frame->frame);
  CmMessagePutInt      (message, frame->GTimeS);
  CmMessagePutInt      (message, frame->GTimeN);
  CmMessagePutInt      (message, 0);
  CmMessagePutInt      (message, nBytes);
  CmMessagePutExtArray (message, CmMessageChar, nBytes, buf);
  CmMessageSetType (message, "FdFrame");

  return(nBytes);
}
/*--------------------------------------------------------------FdSendFrame---*/
void FdSendFrame (char * cmName,
		  const char *destination,
		  struct FrameH *frame,
		  int debugPeriod)
/*----------------------------------------------------------------------------*/
{
  int status;
  CmMessage message;
  struct timeval tv;
  double currentTime, currentTime2;
  char   serData[256]; 
  static int lastGPS = 0, size = 0;
  static double sendingTime = 0., latency = 0.;

  if (frame == NULL) return;
 
  /*---------------------add statistic information about the previous frame---*/
  sprintf(serData,"LastGPS %d Latency %.1f Sending_Time %.1f nBytes %d",
	  lastGPS, latency, sendingTime, size); 
  FrSerDataNew(frame, cmName, frame->GTimeS, serData, 1./ frame->dt);

  /*-------------------------- send frame and measure time need to send it---*/
  gettimeofday(&tv, NULL);
  currentTime = tv.tv_sec + ( tv.tv_usec * 1.0e-6 );

  message = CmMessageNew ();
  size = FdPutFrameInMessage(message, frame, 9);
  status = CmMessageSend(message, (char *) destination);
  CmMessageDelete(message);

  if (status == 0) {
    printf( "Could not send frame GPS%d to %s latency=%.1f nBytes=%d\n", 
	    frame->GTimeS, destination, latency, size);
    FrameFree(frame);

    /*------- try to reconnect in case of problem---*/
    CmMessageCloseServer(cmName);
    CmConnectCleanup();
    if( !CmMessageOpenServer (cmName))
         {printf(" Cannot reopen Cm\n");}
    else {printf(" Reopen server with success\n");}

    return;}
 
  gettimeofday(&tv, NULL);
  currentTime2 = tv.tv_sec + ( tv.tv_usec * 1.0e-6 );
  sendingTime = currentTime2 - currentTime;
  latency = currentTime - FRGPSOFF - FRGPSTAI + frame->ULeapS - frame->GTimeS;
  if(frame->GTimeS % debugPeriod*frame->dt == 0)
    printf( "FdSendFrame>%d Latency=%.1f sending time=%.1f seconds size=%d\n", 
	    frame->GTimeS, latency, sendingTime, size);

  lastGPS = frame->GTimeS;

  FrameFree(frame);
 
  return;
}
